# -*- coding: utf-8 -*-
"""
    eve-swagger.paths
    ~~~~~~~~~~~~~~~~~
    swagger.io extension for Eve-powered REST APIs.

    :copyright: (c) 2015 by Nicola Iarocci.
    :license: BSD, see LICENSE for more details.
"""
from collections import OrderedDict
from flask import current_app as app


def definitions():
    definitions = OrderedDict()
    for rd in app.config['DOMAIN'].values():
        title = rd['item_title']
        definitions[title] = _object(rd['schema'])

    return definitions


def _object(schema):
    props = {}
    required = []
    for field, rules in schema.items():
        if rules.get('required') is True:
            required.append(field)

        props[field] = _type_and_format(rules)

    field_def = {}
    field_def['type'] = 'object'
    field_def['properties'] = props
    if len(required):
        field_def['required'] = required
    return field_def


def _type_and_format(rules):
    resp = {}
    map = {
        'dict': ('object',),
        'list': ('array',),
        'objectid': ('string', 'objectid'),
        'datetime': ('string', 'date-time'),
    }

    eve_type = rules['type']
    type = map.get(eve_type, (eve_type,))

    resp['type'] = type[0]
    if type[0] == 'object':
        # we don't support 'valueschema' rule
        if 'schema' in rules:
            resp.update(_object(rules['schema']))
    elif type[0] == 'array':
        type = 'array'
        if 'schema' in rules:
            resp['items'] = _object(rules['schema'])
        else:
            # 'items' is mandatory for swagger, we assume it's a list of
            # strings
            resp['items'] = {'type': 'string'}
    else:
        try:
            resp['format'] = type[1]
        except IndexError:
            pass

    return resp




# -*- coding: utf-8 -*-
"""
    eve-swagger.objects
    ~~~~~~~~~~~~~~~~~~~
    swagger.io extension for Eve-powered REST APIs.

    :copyright: (c) 2015 by Nicola Iarocci.
    :license: BSD, see LICENSE for more details.
"""
from collections import OrderedDict
from eve.utils import api_prefix
from flask import request, current_app as app

import eve_swagger
from .validation import validate_info


def info():
    validate_info()

    cfg = app.config[eve_swagger.INFO]

    def node(parent, cfg, key):
        value = cfg.get(key)
        if value:
            parent[key] = cfg[key]

    info = OrderedDict()
    node(info, cfg, 'title')
    node(info, cfg, 'description')
    node(info, cfg, 'termsOfService')
    node(info, cfg, 'contact')
    node(info, cfg, 'license')
    node(info, cfg, 'version')

    return info


def host():
    # TODO should probably return None if 'host' has not been set as the host
    # is optional in swagger.
    return app.config.get(eve_swagger.HOST) or request.host


def base_path():
    return api_prefix()


def schemes():
    scheme = request.url.split(':')[0]
    return [scheme] if scheme in ['http', 'https', 'ws', 'wss'] else None


def consumes():
    return ['application/json']


def produces():
    produces = []
    if app.config.get('XML', True):
        produces.append('application/xml')
    if app.config.get('JSON', True):
        produces.append('application/json')
    return produces if produces else None


def definitions():
    pass


def parameters():
    pass


def responses():
    pass


def security_definitions():
    pass


def security():
    pass


def tags():
    pass


def external_docs():
    pass


def swagger_cfg(swagger_key):
    return app.config[eve_swagger.SWAGGER]



# -*- coding: utf-8 -*-
"""
    eve-swagger.swagger
    ~~~~~~~~~~~~~~~~~~~
    swagger.io extension for Eve-powered REST APIs.

    :copyright: (c) 2015 by Nicola Iarocci.
    :license: BSD, see LICENSE for more details.
"""
from collections import OrderedDict
from flask import Blueprint, jsonify

from .objects import info, host, base_path, schemes, consumes, produces, \
    definitions, parameters, responses, security_definitions, security, \
    tags, external_docs
from .paths import paths


swagger = Blueprint('eve_swagger', __name__)


@swagger.route('/api-docs')
def index():
    def node(parent, key, value):
        if value:
            parent[key] = value

    root = OrderedDict()
    root['swagger'] = '2.0'
    node(root, 'info', info())
    node(root, 'host', host())
    node(root, 'basePath', base_path())
    node(root, 'schemes', schemes())
    node(root, 'consumes', consumes())
    node(root, 'produces', produces())
    node(root, 'paths', paths())
    node(root, 'definitions', definitions())
    node(root, 'parameters', parameters())
    node(root, 'responses', responses())
    node(root, 'securityDefinitions', security_definitions())
    node(root, 'security', security())
    node(root, 'tags', tags())
    node(root, 'externalDocs', external_docs())

    return jsonify(root)



#!/usr/bin/env python

from setuptools import setup, find_packages

DESCRIPTION = ("Swagger extension for Eve powered RESTful APIs")
LONG_DESCRIPTION = open('README.rst').read()

setup(
    name='Eve-Swagger',
    version='0.0.4',
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    author='Nicola Iarocci',
    author_email='nicola@nicolaiarocci.com',
    url='http://github.com/nicolaiarocci/cerberus',
    license=open('LICENSE').read(),
    platforms=["any"],
    packages=find_packages(),
    install_requires=['eve'],
    keywords=['swagger', 'eve', 'rest', 'api'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
)



# -*- coding: utf-8 -*-
"""
    eve-swagger.paths
    ~~~~~~~~~~~~~~~~~
    swagger.io extension for Eve-powered REST APIs.

    :copyright: (c) 2015 by Nicola Iarocci.
    :license: BSD, see LICENSE for more details.
"""
from collections import OrderedDict
from flask import current_app as app

# TODO consider adding at least a 'schema' property to response objects
# TODO take auth into consideration

def paths():
    paths = OrderedDict()
    for rd in app.config['DOMAIN'].values():
        methods = rd['resource_methods']
        if methods:
            url = '/%s' % rd['url']
            paths[url] = _resource(rd, methods)

        methods = rd['item_methods']
        if methods:
            item_id = '%sId' % rd['item_title'].lower()
            url = '/%s/{%s}' % (rd['url'], item_id)
            paths[url] = _item(rd, methods, item_id)
    return paths


def _resource(rd, methods):
    item = OrderedDict()
    if 'GET' in methods:
        item['get'] = get_response(rd)
    if 'POST' in methods:
        item['post'] = post_response(rd)
    if 'DELETE' in methods:
        item['delete'] = delete_response(rd)
    return item


def _item(rd, methods, item_id):
    item = OrderedDict()
    if 'GET' in methods:
        item['get'] = getitem_response(rd, item_id)
    if 'PUT' in methods:
        item['put'] = put_response(rd, item_id)
    if 'PATCH' in methods:
        item['patch'] = patch_response(rd, item_id)
    if 'DELETE' in methods:
        item['delete'] = deleteitem_response(rd, item_id)
    return item


def get_response(rd):
    title = rd['resource_title']
    return OrderedDict([
        ('summary', 'Retrieves one or more %s' % title),
        ('responses', {'200': {'description': 'An array of %s' % title}})
    ])


def post_response(rd):
    return OrderedDict([
        ('summary', 'Stores one or more %s' % rd['resource_title']),
        ('responses', {'200':
                       {'description': 'operation has been successful'}})
    ])


def delete_response(rd):
    return OrderedDict([
        ('summary', 'Deletes all %s' % rd['resource_title']),
        ('responses', {'200':
                       {'description': 'operation has been successful'}}),
    ])


def getitem_response(rd, item_id):
    title = rd['item_title']
    return OrderedDict([
        ('summary', 'Retrieves a %s document' % title),
        ('responses', {
            '200': {
                'description': '%s document fetched successfully' % title
            }
        }),
        ('parameters', [id_parameter(item_id, rd)])
    ])


def put_response(rd, item_id):
    title = rd['item_title']
    return OrderedDict([
        ('summary', 'Replaces a %s document' % title),
        ('responses', {
            '200': {
                'description': '%s document replaced successfully' % title
            }
        }),
        ('parameters', [id_parameter(item_id, rd)])
    ])


def patch_response(rd, item_id):
    title = rd['item_title']
    return OrderedDict([
        ('summary', 'Updates a %s document' % title),
        ('responses', {
            '200': {
                'description': '%s document updated successfully' % title
            }
        }),
        ('parameters', [id_parameter(item_id, rd)])
    ])


def deleteitem_response(rd, item_id):
    title = rd['item_title']
    return OrderedDict([
        ('summary', 'Deletes a %s document' % title),
        ('responses', {
            '200': {
                'description': '%s document deleted successfully' % title
            }
        }),
        ('parameters', [id_parameter(item_id, rd)])
    ])


def id_parameter(item_id, rd):
    return OrderedDict([
        ('in', 'path'),
        ('name', item_id),
        ('required', True),
        ('description', 'ID of the %s' %
         rd['item_title']),
        ('type', 'string'),
        ('format', 'objectid')
    ])
